package com.biganto.visual.roompark.presentation.screen.estate

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.EstateInteractor
import com.biganto.visual.roompark.domain.model.FeatureModel
import com.biganto.visual.roompark.domain.model.PlanPresetModel
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject
import javax.inject.Named

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class EstateScreenPresenter @Inject constructor(
    private val interactor: EstateInteractor,
    @Named(SELECTED_ESTATE_ID_KEY) private val estateId:Int
)
    : BigantoBasePresenter<EstateScreen, EstateScreenViewState>() {

    private var planList:List<PlanPresetModel>? = null

    override fun defaultErrorViewStateHandler() =
        {e: ExceptionString -> EstateScreenViewState.SomeError(e) }

    private fun getPlan(plan:PlanPresetModel): Observable<EstateScreenViewState> =
        interactor.getPlan(plan)
            .map<EstateScreenViewState> {EstateScreenViewState.LoadPlan(it)}

    override fun bindIntents() {

        val prefetchCards = interactor.getEstate(estateId)
            .map { EstateScreenViewState.LoadEstate(it) }

        val fetchPlans = interactor.getPlanTypes(estateId)
            .doOnNext { planList = it.toList() }
            .map { EstateScreenViewState.LoadPlanTypes(it) }

        val fetchPlan = intent(EstateScreen::planTypesTabSelected)
            .map { planList?.get(it) }
            .flatMap{ interactor.getPlan(it)
                .map<EstateScreenViewState> {plan -> EstateScreenViewState.LoadPlan(plan) }
                .startWith(Observable.just<EstateScreenViewState>(EstateScreenViewState.PlanTypeSelected(it)))
            }

        val switchSizes = intent(EstateScreen::switchSizes)
            .map {pair ->
                val plan = planList?.first{it.planId == pair.first}
                plan?.switchFeature<FeatureModel.Sizes>(pair.second)
                plan
            }.flatMap(::getPlan)

        val switchFurn = intent(EstateScreen::switchFurniture)
            .map {pair ->
                val plan = planList?.first{it.planId == pair.first}
                plan?.switchFeature<FeatureModel.Furniture>(pair.second)
                plan
            }.flatMap(::getPlan)

        val switchWalls = intent(EstateScreen::switchWalls)
            .map {pair ->
                val plan = planList?.first{it.planId == pair.first}
                plan?.switchFeature<FeatureModel.Walls>(pair.second)
                plan
            }.flatMap(::getPlan)

        val switchElectric = intent(EstateScreen::switchElectric)
            .map {pair ->
                val plan = planList?.first{it.planId == pair.first}
                plan?.switchFeature<FeatureModel.Electric>(pair.second)
                plan
            }.flatMap(::getPlan)

        val state = restoreStateObservable
            .mergeWith(prefetchCards)
            .mergeWith(fetchPlans)
            .mergeWith(fetchPlan)
            .mergeWith(switchElectric)
            .mergeWith(switchFurn)
            .mergeWith(switchSizes)
            .mergeWith(switchWalls)
            .doOnError{ Timber.e(it)}
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(EstateScreenViewState::class.java), EstateScreen::render)
    }

}